// Game System
var Reversi = (function() {

    /* 定数 */
    var CIRCUMFEFENCE = [[-1, -1], [0, -1], [1, -1], [-1, 0], [1, 0], [-1, 1], [0, 1], [1, 1]], // 8方向
        STONESIZE     = 40, // 石のサイズ
        NONE          = 0,  // マス目に石が置いてない状態
        WHITE         = 1,  // マス目に白石が置いてある状態
        BLACK         = 2;  // マス目い黒石が置いてある状態

    /* 変数 */
    var board,    // 盤面の状態を保持する配列(2次元配列にしてリバーシの盤面を用意する)
        turn,     // 現在どちらのターンなのかを保持する。
        finish,   // ゲームが終了したかどうかのフラグ 
        blackCnt, // 現在の黒石の数を保持する変数 
        whiteCnt, // 現在の白石の数を保持する変数 
        ctx,      // canvasのコンテキスト 
        info;     // 対戦中の情報を表示させる領域  

    /* 変数の初期化メソッド */
    var initVar = function() {
        board      = []; 
        info       = document.getElementById('reversiInfo'); 
        turn       = BLACK; // リバーシは黒から始まるゲームなので黒で初期化する
        finish     = false;
        blackCnt   = null;
        whiteCnt   = null;
        ctx        = null;
    };

    /* 盤面の初期化メソッド */
    var initBoard = function() {

        board = [];

        // boardの中身を石がない状態で初期化する
        for ( var y = 0; y < 8; y++ ) {
            board[y] = [];
            for ( var x = 0; x < 8; x++ ) {
                board[y][x] = NONE;
            }
        }

        // 白と黒の石を真ん中にセットしリバーシの初期状態を作る
        board[3][3] = WHITE;
        board[4][4] = WHITE;
        board[3][4] = BLACK;
        board[4][3] = BLACK;
    };

    /* 石を書くメソッド */
    var drawStone = function(x, y, color) {
        ctx.beginPath();
        ctx.arc(x, y, 20, 0, Math.PI*2, false);
        ctx.fillStyle = color;
        ctx.fill();
    };

    /* 盤面を描画するメソッド */
    var drawBoard = function() {

        ctx.beginPath();

        // 画面を緑で埋める
        ctx.fillStyle = '#008800';
        ctx.fillRect(0, 0, 400, 400);
	
        // 線を描画する	
        ctx.strokeStyle = '#000000';
        ctx.lineWidth = 0.5;
        for ( var i = 0; i < 9; i++ ) {
            ctx.moveTo(i * STONESIZE, 0);
            ctx.lineTo(i * STONESIZE, 8 * STONESIZE);
            ctx.moveTo(0, i * STONESIZE);
            ctx.lineTo(8 * STONESIZE, i * STONESIZE);
        }
        ctx.stroke();
    };

    /* 盤面から石がはみ出たかどうかチェックするメソッド */
    var isOut = function (x, y) {
        return ( x < 0 || x > 7 || y < 0 || y > 7 );
    };

    /* 列単位で石を置けるかどうかの判定メソッド */
    var lineCheck = function(direction, x, y, stone) {

        var queue = '';

        // 盤面からはみでるまでループさせる
        while ( 1 ) {
            // directionで指定した方向へ1マスづつ進む
            x += direction[0];
            y += direction[1];

            // 盤面からはみでたらbreakで抜ける
            if ( isOut(x, y) )
                break;

            // queueに自分がタッチした位置から盤面をはみでるまでdirectionで指定した方向の盤面の状態を格納していく
            queue += board[y][x];
        }

        // データを詰めたキューの中身を解析する
        // 自分と反対の色の石が連続して続いた後に自分の石の色が現れた場合はそこに石を置く事が可能なのでtrueを返す。置けない場合はfalseを返す。
        return ( queue.search(stone==WHITE ? /2+1/ : /1+2/) == 0 );
    };

    /* 石を置けるかどうかの判定メソッド */
    var isAbleToPut = function(x, y, stone) {

    // タッチされた場所が盤面の内側で石が置いてなければそこに石を置ける可能性がある
    if ( !isOut(x, y) && (board[y][x] == NONE) ) {
        for (var i = 0; i < CIRCUMFEFENCE.length; i++) {
            var direction = CIRCUMFEFENCE[i];
            // 石を置きたい場所の周り八方向に対してlineCheck()を実行する
            // もし一方向でもlineCheck()がtrueを返してくれればそこには石を置けるという事がわかる
            if ( lineCheck(direction, x, y, stone) )
                return true;
            }
        }
        // 八方向ともlineCheck()がtrueを返してくれなければそこには石を置けない
        return false;
    };

    /* 自分のターンが続くかチェックするメソッド */
    var continueOwnTurn = function(stone) {

        for ( var y = 0; y < 8; y++ ) {
            for ( var x = 0; x < 8; x++ ) {
                // 盤面の全てのマスに対してisAbleToPut()を実行する事によって置ける場所があるかどうかが分かる
                if( isAbleToPut(x, y, stone) )
                    return true;
                }
            }
        // どのマスにも置けなければfalseを返す
	    return false;
    };

    /* ひっくり返すメソッド */
    var turnover = function(x, y, stone) {
        // 盤面に石を置く
        board[y][x] = stone;

        // 置いた場所の周り八方向をチェックする
        for ( var k = 0; k < CIRCUMFEFENCE.length; k++ ) {
            var direction = CIRCUMFEFENCE[k];

            // 一方向づつひっくり返す石があるかチェックする
            if ( lineCheck(direction, x, y, stone) ) {

                // ひっくり返す石があった場合ひっくり返す処理を実行する
                for ( var x_ = x, y_ = y;; ) {
                    // 石を置いた所から一個づつ盤面の外側に向かって進む
                    x_ += direction[0];
                    y_ += direction[1];
	    
                    // 盤面の外にでたり、自分の色とぶつかったらbreakで抜ける
                    if ( isOut(x_, y_) )
                        break;
                    if ( stone == BLACK && board[y_][x_] == BLACK )
                        break;
                    if ( stone == WHITE && board[y_][x_] == WHITE )
                        break;

                    // 自分の石の色に置き換える 
                    board[y_][x_] = stone;
                }
            }
        }
    };

    /* ゲームの描画メソッド */
    var drawGame = function() {

        blackCnt = 0;
        whiteCnt = 0;

        // 古い画面を一度画面を消去する
        ctx.clearRect(0, 0, ctx.canvas.width, ctx.canvas.height);

        // 盤面の描画
        drawBoard();

        // 配列に入っているデータを元に石を盤面に表示させる
        for ( var y = 0; y < 8; y++ ) {
            for ( var x = 0; x < 8; x++ ) {
                if ( board[y][x] ) {
                    drawStone(x*STONESIZE + STONESIZE/2, y*STONESIZE + STONESIZE/2, (board[y][x]==BLACK ? '#000000' : '#FFFFFF') );
                    // 石のカウント
                    board[y][x] == BLACK ? blackCnt++ : whiteCnt++;
                }
            }
        }

        // 情報表示エリアの色を変更し石の数を描画させる
        info.style.backgroundColor = (turn == BLACK) ? '#000000' : '#FFFFFF';
        info.innerHTML = ' BLACK:' + blackCnt + ' WHITE:' + whiteCnt;

    };

    /* 勝敗の判定メソッド */
    var judge = function() {
        // 白または黒の石の置き場が無くなった場合ゲームが終了する
        if ( !finish && (!continueOwnTurn(BLACK) && !continueOwnTurn(WHITE)) ) {
            Ext.Msg.alert('判定', (whiteCnt == blackCnt ? '引き分けです' : whiteCnt > blackCnt ? '白の勝ちです' : '黒の勝ちです') );
            finish = true;
        }

    };

    return {

        /* メインの処理 */
        start: function() {

            // 初期化
            initVar();
            initBoard();

            var canavs = document.getElementById('canvas');
            ctx  = canvas.getContext('2d');

            // ゲーム画面を初期状態で一度描画
            drawGame();

            // クリックした時の処理
            canavs.addEventListener('click', function(e) {

                // タッチされた場所がリバーシ版のどこかを計算する
                var x = Math.floor(e.offsetX / STONESIZE);
                var y = Math.floor(e.offsetY / STONESIZE);

                // タッチされた場所に石を置けるようなら処理を行う
                if ( isAbleToPut(x, y, turn) ) {
                    // タッチされた場所に石を置く
                    board[y][x] = turn;
                    // 石をひっくり返す
                    turnover(x, y, turn);

                    // 相手が石を置ける状態なら相手のターンにする
                    if ( continueOwnTurn((turn==BLACK ? WHITE : BLACK)) ){
                        turn = (turn == BLACK) ? WHITE : BLACK;
                    }

                    // 画面を描画し直す
                    drawGame();

                    // 勝敗の判定
                    judge();
                }
          
            });
        }
    };
})();


// Sencha Touch
Ext.setup({

    // ホームスクリーンブックマークした時のアイコン
    icon: './img/icon.png',
    // アイコンに光沢をつける
    glossOnIcon: true,
    // ホームスクリーンブックマークから起動した時のスタートアップ画像
    phoneStartupScreen: './img/phone_startup.png',

    onReady: function(){

        // パネル1 (TOP画面)
        var panel1 = new Ext.Panel({

            fullscreen: true,

            layout: {
                type: 'vbox',
                pack: 'center'
            },

            items: [
                // TOP画像
                {html: '<img src="./img/title.png"></img>', width:320, height:80},
                // ボタン1(1P vs CPU)
                new Ext.Button({cls:'startBtn1', text:'先攻', width:'120px', height:'50px', handler:function(){panel2.show('fade'); }}), //ボタンが押されたらパネル2を表示させる
                // ボタン2(CPU vs 1P)
                new Ext.Button({cls:'startBtn2', text:'後攻', width:'120px', height:'50px', handler:function(){panel2.show('fade'); }}), //ボタンが押されたらパネル2を表示させる 
                // ボタン3(1P vs 2P)
                new Ext.Button({cls:'startBtn3', text:'対人戦',  width:'250px', height:'50px', handler:function(){panel2.show('fade'); Reversi.start();}})  //ボタンが押されたらパネル2を表示させる
            ]
        });

        // パネル2 (ゲーム画面、設定画面、アプリ説明画面を配置するタブパネル)
        var panel2 = new Ext.TabPanel({

            fullscreen: true,

            defaults: {
                iconMast: true,
                ui:       'plain'
            },

            // 画面上部にツールバーを追加
            dockedItems: new Ext.Toolbar({
                dock:  'top',
                title: 'Reversi',
                ui: 'light',
                items:[
                    // TOPボタンが押されたらpanel2を隠す
                    {text: 'TOP', handler:function(){panel2.hide();}},
                    {xtype: 'spacer'},
                    {iconMask: true, iconCls: 'refresh', handler:function(){Reversi.start();}}
                ]
            }),

            // タブの設定
            tabBar: {
                dock:  'bottom',
                ui:    'light',
                layout: {pack:'center'}
            },

            // タブの切り替え方法(スライド)
            cardSwitchAnimation: {
                type: 'slide'
            },

            // 各ページの設定
            items: [
                {iconCls:'favorites', html:'<div style="width:100%; text-align:center;"><canvas id="canvas" width="320" height="320"></canvas><div id="reversiInfo" style="font-family: メイリオ;width:320px;height:50px; position:relative; top:-5px; background-color:#000000; margin-left:auto;margin-right:auto; padding: 13px 0 0 0; text-shadow:0 0 2px #9ff,0 0 4px #622,0 0 6px #3ff,0 0 8px #0ff,0 0 10px #9cc,0 0 12px #099,0 0 14px #066,0 0 16px #033; font-weight:bold;"></div></div>'},
                {iconCls:'settings',  html:'設定画面'},
                {iconCls:'info',      html:'アプリの説明画面'}
            ]

        });

        // 初期状態ではパネル2を隠して、パネル1を表示する。
        panel2.hide();
        panel1.show();
    }
});
